package com.landray.kmss.sys.attachment.io;

import java.io.InputStream;

public class EncryptionInputStream extends AbstractInputStream
{
    private boolean noEncryption;
    
    public EncryptionInputStream(final InputStream originStream) throws Exception {
        super(originStream);
    }
    
    public EncryptionInputStream(final InputStream originStream, final int encryptionMode) throws Exception {
        super(originStream, encryptionMode);
    }
    
    @Override
    protected void init() throws Exception {
        this.noEncryption = (this.cryptionMode != 1 && this.cryptionMode != 3 && this.cryptionMode != 4);
        if (this.noEncryption) {
            return;
        }
        if (this.cryptionMode == 3) {
            this.initAES();
        }
        if (this.cryptionMode == 4) {
            this.initSM4();
        }
        if (this.cryptionMode == 1) {
            this.additionalBytes = EncryptionInputStream.markBytes;
        }
        else if (this.cryptionMode == 3) {
            this.additionalBytes = EncryptionInputStream.aesMarkBytes;
        }
        else if (this.cryptionMode == 4) {
            this.additionalBytes = EncryptionInputStream.sm4MarkBytes;
        }
        else {
            this.additionalBytes = EncryptionInputStream.aesMarkBytes;
        }
    }
    
    @Override
    protected void format(final byte[] b, final int off, final int len) {
        if (this.noEncryption) {
            return;
        }
        for (int i = off; i < len; ++i) {
            if (this.cryptionMode == 3) {
                b[i] = this.aesCrypt.encrypt(b, i, 1)[0];
            }
            else if (this.cryptionMode == 4) {
                b[i] = this.sm4Crypt.encrypt(b, i, 1)[0];
            }
            else if (b[i] == 127) {
                b[i] = -128;
            }
            else {
                final int n = i;
                ++b[n];
            }
        }
    }
    
    @Override
    protected int format(final int i) {
        if (this.noEncryption) {
            return i;
        }
        if (i == -1) {
            return -1;
        }
        if (this.cryptionMode == 3) {
            final byte[] b = this.aesCrypt.encrypt(new byte[] { (byte)i });
            return b[0];
        }
        if (this.cryptionMode == 4) {
            final byte[] b = this.sm4Crypt.encrypt(new byte[] { (byte)i });
            return b[0];
        }
        if (i == 255) {
            return 0;
        }
        return i + 1;
    }
}
